from .node import Node
from .vector_elements import Rectangle, Text, TextField, Image, UnknownElement
from ..utils import download_image
from pathlib import Path


class Frame(Node):
    def __init__(self, node, output_path, figma_file, parent=None):
        super().__init__(node)

        self.parent = parent

        self.width, self.height = self.size()
        self.x, self.y = self.position()
        self.bg_color = self.color()

        self.border_radius = self.get_border_radius()
        self.shadow = self.get_shadow()

        self.counter = {}

        self.figma_file = figma_file

        self.output_path: Path = output_path
        self.assets_path: Path = output_path / "assets"

        self.output_path.mkdir(parents=True, exist_ok=True)
        self.assets_path.mkdir(parents=True, exist_ok=True)

        self.elements = [
            self.create_element(child) for child in self.children if Node(child).visible
        ]

    def create_element(self, element):
        element_name = element["name"].strip().lower()
        element_type = element["type"].strip().lower()

        if element_type == "frame" or element_type == "group":
            return Frame(
                element,
                figma_file=self.figma_file,
                output_path=self.output_path,
                parent=self,
            )
        elif element_name == "textfield":
            return TextField(element, self)

        elif element_type == "rectangle" and element["fills"][0]["type"] == "IMAGE":
            return self.handle_image_element(element)
        if element_name == "rectangle" or element_type == "rectangle":
            return Rectangle(element, self)
        elif element_type == "text":
            return Text(element, self)

        else:
            return UnknownElement(element, self)

    def handle_image_element(self, element):
        self.counter[Image] = self.counter.get(Image, 0) + 1
        item_id = element["id"]
        image_url = self.figma_file.get_image(item_id)
        image_path = self.assets_path / f"image_{self.counter[Image]}.png"
        download_image(image_url, image_path)

        image_path = image_path.relative_to(self.assets_path)

        return Image(element, self, image_path, id_=f"{self.counter[Image]}")

    @property
    def children(self):
        return self.node.get("children")

    def color(self) -> str:
        """Returns HEX form of element RGB color (str)"""
        try:
            color = self.node["fills"][0]["color"]
            r, g, b, *_ = [int(color.get(i, 0) * 255) for i in "rgba"]

            return f"#{r:02X}{g:02X}{b:02X}"

        except Exception:
            return "transparent"

    def size(self) -> tuple:
        """Returns element dimensions as width (int) and height (int)"""
        bbox = self.node["absoluteBoundingBox"]
        width = bbox["width"]
        height = bbox["height"]
        return int(width), int(height)

    def position(self):
        # Returns element coordinates as x (int) and y (int)
        bbox = self.node["absoluteBoundingBox"]
        x = bbox["x"]
        y = bbox["y"]

        #
        if self.parent is None:
            x = 0
            y = 0
        else:
            parent_bbox = self.parent.node["absoluteBoundingBox"]
            x -= parent_bbox["x"]
            y -= parent_bbox["y"]

        return int(x), int(y)

    def get_border_radius(self) -> int:
        if "cornerRadius" in self.node:
            return self.node["cornerRadius"]
        else:
            return 0

    def get_shadow(self) -> dict:
        """Returns the shadow properties as a dictionary."""
        try:
            for effect in self.node.get("effects", []):
                if effect["type"] == "DROP_SHADOW" and effect["visible"]:
                    color = effect["color"]
                    r, g, b, a = [int(color.get(k, 0) * 255) for k in "rgba"]
                    shadow_color = f"#{r:02X}{g:02X}{b:02X}"
                    offset = effect["offset"]
                    blur = effect.get("radius", 0)
                    spread = effect.get("spread", 0)  # Optional
                    return {
                        "color": shadow_color,
                        "offset_x": int(offset["x"]),
                        "offset_y": int(offset["y"]),
                        "blur": int(blur),
                        "spread": int(spread),
                    }
        except KeyError:
            pass
        return None  # No shadow

    def to_code(self):
        # print(self.font_urls)
        # border_radius = self.border_radius
        # border_radius_str = (
        #     f"border_radius=ft.border_radius.all({border_radius[0]})"
        #     if all(r == border_radius[0] for r in border_radius)
        #     else f"border_radius=ft.border_radius.only("
        #     f"topLeft={border_radius[0]}, "
        #     f"topRight={border_radius[1]}, "
        #     f"bottomRight={border_radius[2]}, "
        #     f"bottomLeft={border_radius[3]})"
        # )

        # shadow to Flet-compatible string
        shadow_str = ""
        if self.shadow:
            shadow = self.shadow
            shadow_str = f"""
            shadow=ft.BoxShadow(
                spread_radius={shadow["spread"]},
                blur_radius={shadow["blur"]//5},
                offset=ft.Offset({shadow["offset_x"]}, {shadow["offset_y"]}),
                color="{shadow["color"]}"
            ),
            """

        # Generate code for all child elements
        children_code = ",\n".join(child.to_code() for child in self.elements)
        if children_code:
            return f"""
            ft.Container(
                left={self.x},
                top={self.y},
                width={self.width},
                height={self.height},
                border_radius={self.border_radius},
                
                {shadow_str}
                bgcolor="{self.bg_color}",
                content=ft.Stack([
                    {children_code},
                ])
            )
        """
        else:
            return f"""
            ft.Container(
                width={self.width},
                height={self.height},
                bgcolor="{self.bg_color}",
            )
            """


class Group(Frame):
    def __init__(self, node):
        super().__init__(node)


class Component(Frame):
    def __init__(self, node):
        super().__init__(node)


class ComponentSet(Frame):
    def __init__(self, node):
        super().__init__(node)



import io
import requests
from PIL import Image


def get_fonts_urls(font_family):
    # Format the font-family name for URL
    font_family_name = font_family.split()
    # print(font_family_url)
    google_fonts_url = f"https://fonts.googleapis.com/css2?family={font_family_name[0]}"
    # print(google_fonts_url)
    # Fetch the font CSS
    response = requests.get(google_fonts_url)
    if response.status_code == 200:
        css_content = response.text

        # # Extract font file URLs from the CSS and download them
        font_urls = [
            line.split("url(")[-1].split(")")[0].strip('"')
            for line in css_content.splitlines()
            if "url(" in line
        ]
        page_settings = f"{font_family}:{font_urls[0]}"
        return page_settings

    else:
        print(
            f"Failed to fetch font CSS for {font_family}. Status code: {response.status_code}"
        )


def download_image(url, image_path):
    response = requests.get(url)
    content = io.BytesIO(response.content)
    im = Image.open(content)
    im = im.resize((im.size[0] // 2, im.size[1] // 2), Image.LANCZOS)
    with open(image_path, "wb") as file:
        im.save(file)



import re
import flet as ft
from pathlib import Path
from figmaflet.generateUI import UI


# TODO: - Redesign the gui UI, make it more beautiful and add a logo


# Extract file key from figma URL
def extract_file_key(url):

    match = re.search(r"/design/([a-zA-Z0-9]+)", url)
    if match:
        return match.group(1)
    return None


apikey = ft.TextField(label="API Token", border_radius=30, bgcolor="grey100")
file_url = ft.TextField(label="File URL", border_radius=30, bgcolor="grey100")
path = ft.TextField(label="Output PATH", border_radius=30, bgcolor="grey100")


def main(page: ft.Page):
    page.title = "FigmaFlet"
    page.theme_mode = "light"
    page.window.width = 600
    page.spacing = 30
    page.horizontal_alignment = "center"

    def submit_data(e):
        apikey_value = apikey.value.strip()
        url_value = file_url.value.strip()
        path_value = path.value.strip()

        # Extract the file key from the provided URL
        file_url_value = extract_file_key(url_value)

        # Validate input
        if not apikey_value or not file_url_value or not path_value:
            page.open(ft.AlertDialog(content=ft.Text("All fields are required!")))
            return

        # Ensure the path exists
        output_path = Path(path_value)
        if not output_path.exists():
            page.open(ft.AlertDialog(content=ft.Text("Invalid file path!")))
            return

        # Generate the Flet `UI`
        try:
            ui_generator = UI(apikey_value, file_url_value, output_path)
            ui_generator.generate()
            page.open(
                ft.AlertDialog(
                    content=ft.Text(
                        "UI generated successfully!", size=20, color="green"
                    )
                )
            )
        except Exception as ex:
            page.open(
                ft.AlertDialog(content=ft.Text(f"An error occurred: {ex}", color="red"))
            )

    page.add(
        ft.Column(
            [
                ft.Text("FigmaFlet", size=30, weight=ft.FontWeight.BOLD),
                ft.Text(
                    "Generate Flet UIs from Figma Designs",
                    size=18,
                    weight=ft.FontWeight.W_500,
                ),
            ],
            horizontal_alignment="center",
        ),
        ft.Column(
            [
                apikey,
                file_url,
                path,
                ft.ElevatedButton("GENERATE", ft.Icons.UPLOAD, on_click=submit_data),
            ],
            horizontal_alignment="center",
            spacing=25,
        ),
    )


if __name__ == "__main__":
    ft.app(target=main)



from jinja2 import Template
from figmaflet.template import TEMPLATE
from figmaflet.figma.frame import Frame
from figmaflet.figma import endpoints
from figmaflet.figma.vector_elements import Text
from figmaflet.utils import get_fonts_urls
from pathlib import Path


class UI:
    def __init__(self, token: str, file_key: str, local_path: Path):

        self.figma_file = endpoints.Files(token, file_key)
        self.file_data = self.figma_file.get_file()
        self.local_path = local_path

        self.font_families = set()

    def to_code(self):

        # Generate Flet code for each frame
        for f in self.file_data["document"]["children"][0]["children"]:
            frame = Frame(f, figma_file=self.figma_file, output_path=self.local_path)
            # frames.append(frame.to_code())

            # Collect font URLs from frame elements
            self.collect_font_families(frame)

            font_list = [get_fonts_urls(family) for family in self.font_families]
            font_urls = {
                item.split(":")[0]: "https:" + item.split(":")[2] for item in font_list
            }

            # Render the template
            t = Template(TEMPLATE)
            rendered_code = t.render(elements=frame.to_code(), font_urls=font_urls)
            return rendered_code

    def collect_font_families(self, frame):
        for element in frame.elements:
            if isinstance(element, Text):
                self.font_families.add(element.font_family)
            elif isinstance(element, Frame):
                # Recursively collect from nested frames
                self.collect_font_families(element)

    def generate(self):
        code = self.to_code()
        self.local_path.joinpath("main.py").write_text(code, encoding="UTF-8")



import re
import argparse
from pathlib import Path
from figmaflet.generateUI import UI


# def extract_file_key(url):
#     match = re.search(r"/design/([a-zA-Z0-9]+)", url)
#     if match:
#         return match.group(1)
#     return None


def main():
    parser = argparse.ArgumentParser(description="Generate Flet UI from Figma designs.")
    parser.add_argument("--apitoken", required=True, help="Your Figma API token.")
    parser.add_argument("--fileurl", required=True, help="The URL of the Figma file.")
    parser.add_argument(
        "--output", required=True, help="Output file for the generated UI code."
    )

    args = parser.parse_args()

    ui = UI(
        token=args.apitoken,
        file_key=args.fileurl,
        local_path=Path(args.output),
    )
    ui.generate()

    print(f"UI code has been successfully generated and saved to {args.output}.")


if __name__ == "__main__":
    main()



TEMPLATE = """
# This file was generated by FigmaFlet by Benit Mulindwa
# https://github.com/Benitmulindwa/FigmaFlet

import flet as ft

def main(page: ft.Page):
    page.padding=0
    page.fonts={{font_urls}}
    page.add(
       ft.Stack([
        {{ elements }}
        ])
    )
ft.app(target=main)
"""



""" Utility classes and functions for Figma API endpoints.
"""

import requests

_token = "FIGMA-API"


class Files:
    """https://www.figma.com/developers/api#files-endpoints"""

    API_ENDPOINT_URL = "https://api.figma.com/v1"

    def __init__(self, token, file_key):
        self.token = token
        self.file_key = file_key

    def __str__(self):
        return f"Files {{ Token: {self.token}, File: {self.file_key} }}"

    def get_file(self) -> dict:
        try:
            response = requests.get(
                f"{self.API_ENDPOINT_URL}/files/{self.file_key}",
                headers={"X-FIGMA-TOKEN": self.token},
            )
        except ValueError:
            raise RuntimeError("Invalid Input. Please check your input and try again.")
        except requests.ConnectionError:
            raise RuntimeError("Flet Designer requires internet access to work.")
        else:
            return response.json()

    def get_image(self, item_id) -> str:
        response = requests.get(
            f"{self.API_ENDPOINT_URL}/images/{self.file_key}?ids={item_id}&scale=2",
            headers={"X-FIGMA-TOKEN": self.token},
        )
        return response.json()["images"][item_id]




# This file was generated by FigmaFlet by Benit Mulindwa
# https://github.com/Benitmulindwa/FigmaFlet

import flet as ft

def main(page: ft.Page):
    page.padding=0
    page.add(
       ft.Stack([
        
            ft.Container(
                left=0,
                top=0,
                width=1440,
                height=900,
                border_radius=0,
                
                
                bgcolor="#FFFFFF",
                content=ft.Stack([
                    
        ft.Container(
            left=0.0,
            top=0.0,
            width=1440.0,
            height=900.0,
            border_radius=None,
            bgcolor="#000000",)
,

            ft.Container(
                left=80,
                top=37,
                width=1222,
                height=29,
                border_radius=0,
                
                
                bgcolor="transparent",
                content=ft.Stack([
                    
            ft.Container(
                left=0,
                top=0,
                width=1222,
                height=29,
                border_radius=0,
                
                
                bgcolor="transparent",
                content=ft.Stack([
                    
        ft.Container(
            content=ft.Text(value="FletEditor", size=24.0, color="#FFFFFF",weight="w700",text_align=ft.TextAlign.LEFT),
            left=0.0,
            top=0.0,
            )
        ,

            ft.Container(
                left=408,
                top=3,
                width=463,
                height=22,
                border_radius=0,
                
                
                bgcolor="transparent",
                content=ft.Stack([
                    
        ft.Container(
            content=ft.Text(value="Main", size=18.0, color="#FFFFFF",weight="w500",text_align=ft.TextAlign.LEFT),
            left=0.0,
            top=0.0,
            )
        ,

        ft.Container(
            content=ft.Text(value="About", size=18.0, color="#FFFFFF",weight="w500",text_align=ft.TextAlign.LEFT),
            left=86.0,
            top=0.0,
            )
        ,

        ft.Container(
            content=ft.Text(value="Program", size=18.0, color="#FFFFFF",weight="w500",text_align=ft.TextAlign.LEFT),
            left=183.0,
            top=0.0,
            )
        ,

        ft.Container(
            content=ft.Text(value="Price", size=18.0, color="#FFFFFF",weight="w500",text_align=ft.TextAlign.LEFT),
            left=303.0,
            top=0.0,
            )
        ,

        ft.Container(
            content=ft.Text(value="Contact", size=18.0, color="#FFFFFF",weight="w500",text_align=ft.TextAlign.LEFT),
            left=390.0,
            top=0.0,
            )
        ,
                ])
            )
        ,

            ft.Container(
                left=1154,
                top=4,
                width=68,
                height=20,
                border_radius=0,
                
                
                bgcolor="transparent",
                content=ft.Stack([
                    
ft.Container(
    left=0.0,
    top=0.0,
    width=20.0,
    height=20.0,
    bgcolor="pink")
,

ft.Container(
    left=30.0,
    top=1.0,
    width=18.0,
    height=18.0,
    bgcolor="pink")
,
                ])
            )
        ,
                ])
            )
        ,
                ])
            )
        ,

            ft.Container(
                left=422,
                top=290,
                width=658,
                height=158,
                border_radius=0,
                
                
                bgcolor="transparent",
                content=ft.Stack([
                    
        ft.Container(
            content=ft.Text(value="EDIT LIKE A PRO", size=72.0, color="#FFFFFF",weight="w700",text_align=ft.TextAlign.LEFT),
            left=16.0,
            top=0.0,
            )
        ,

        ft.Container(
            content=ft.Text(value="and make money", size=72.0, color="#FFFFFF",weight="w700",text_align=ft.TextAlign.LEFT),
            left=0.0,
            top=70.0,
            )
        ,
                ])
            )
        ,

        ft.Container(
            content=ft.Text(value="""This is an intensive program for those who \nwant to master the  skills of professional graphic design """, size=18.0, color="#FFFFFF",weight="w400",text_align=ft.TextAlign.CENTER),
            left=501.0,
            top=478.0,
            )
        ,

            ft.Container(
                left=669,
                top=552,
                width=180,
                height=57,
                border_radius=40.0,
                
                
            shadow=ft.BoxShadow(
                spread_radius=0,
                blur_radius=5,
                offset=ft.Offset(0, 0),
                color="#A8FF35"
            ),
            
                bgcolor="#A8FF35",
                content=ft.Stack([
                    
        ft.Container(
            content=ft.Text(value="Order", size=18.0, color="#000000",weight="w700",text_align=ft.TextAlign.CENTER),
            left=63.0,
            top=18.0,
            )
        ,
                ])
            )
        ,
                ])
            )
        
        ])
    )
ft.app(target=main)


class Node:
    def __init__(self, node: dict):
        self.node = node

    @property
    def id(self) -> str:
        return self.node.get("id")

    @property
    def name(self) -> str:
        return self.node.get("name")

    @property
    def visible(self) -> bool:
        """Whether or not the node is visible on the canvas."""
        return self.node.get("visible", True)

    @property
    def type(self) -> str:
        return self.node.get("type")

    @property
    def plugin_data(self):
        return self.node.get("pluginData")

    @property
    def shared_plugin_data(self):
        return self.node.get("sharedPluginData")

    def get(self, key, default=None):
        return self.node.get(key, default)


class Document(Node):
    def __init__(self, node, root="window"):
        super().__init__(node)
        self.root = root

    @property
    def children(self) -> list[dict]:
        return self.node.get("children")


class Canvas(Node):
    def __init__(self, node):
        super().__init__(node)

    @property
    def children(self):
        # TODO: Convert nodes to Node objects before returning a list of them.
        return self.node.get("children")

    @property
    def background_color(self):
        return self.node.get("backgroundColor")

    @property
    def prototype_start_node_id(self) -> str:
        return self.node.get("prototypeStartNodeID")

    @property
    def export_settings(self):
        return self.node.get("exportSettings")

    def generate(self):
        return ""


class Slice(Node):
    def __init__(self, node):
        super().__init__(node)

    @property
    def export_settings(self):

        return self.node.get("exportSettings")

    @property
    def absolute_bounding_box(self):

        return self.node.get("absoluteBoundingBox")

    @property
    def size(self):

        return self.node.get("size")

    @property
    def relative_transform(self):

        return self.node.get("relativeTransform")



from .node import Node


class Vector(Node):
    def __init__(self, node: dict) -> None:
        super().__init__(node)

    def strockes_color(self):

        try:
            stroke = self.node["strokes"][0]
            color = self.node["strokes"][0]["color"]
            r, g, b, *_ = [int(color.get(i, 0) * 255) for i in "rgba"]
            # Extract opacity (default to 1 if not provided)
            opacity = stroke.get("opacity", 1) * self.node.get("opacity", 1)
            return [round(opacity, 2), f"#{r:02X}{g:02X}{b:02X}"]
        except:
            return [1, "transparent"]

    def color(self) -> str:
        """Returns HEX form of element RGB color (str)"""
        fill = self.node["fills"][0]
        try:
            color = self.node["fills"][0]["color"]
            r, g, b, *_ = [int(color.get(i, 0) * 255) for i in "rgba"]
            # Extract opacity (default to 1 if not provided)
            opacity = fill.get("opacity", 1) * self.node.get("opacity", 1)

            return [round(opacity, 2), f"#{r:02X}{g:02X}{b:02X}"]

        except Exception:
            return [1, "transparent"]

    def size(self):
        bbox = self.node["absoluteBoundingBox"]
        width = bbox["width"]
        height = bbox["height"]
        return int(width), int(height)

    def position(self, frame):
        # Returns element coordinates as x (int) and y (int)
        bbox = self.node["absoluteBoundingBox"]
        x = bbox["x"]
        y = bbox["y"]

        frame_bbox = frame.node["absoluteBoundingBox"]
        frame_x = frame_bbox["x"]
        frame_y = frame_bbox["y"]

        x = abs(x - frame_x)
        y = abs(y - frame_y)
        return x, y


# Handled Figma Components
class Rectangle(Vector):
    def __init__(self, node, frame):
        super().__init__(node)
        self.x, self.y = self.position(frame)
        self.width, self.height = self.size()
        self.opacity, self.bg_color = self.color()
        self.gradient = None

    def get_effects(self) -> dict:

        effects = {"shadow": None, "background_blur": None, "gradient": None}
        try:
            for effect in self.get("effects", []):
                if effect["type"] == "DROP_SHADOW" and effect["visible"]:
                    color = effect["color"]
                    r, g, b, *_ = [int(color.get(i, 0) * 255) for i in "rgba"]
                    shadow_color = f"#{r:02X}{g:02X}{b:02X}"

                    offset = effect["offset"]
                    blur = effect.get("radius", 0)

                    effects["shadow"] = {
                        "color": shadow_color,
                        "offset_x": int(offset["x"]) / 2,
                        "offset_y": int(offset["y"]) / 2,
                        "blur": int(blur),
                    }
                elif effect["type"] == "BACKGROUND_BLUR" and effect["visible"]:
                    effects["background_blur"] = {"radius": effect.get("radius", 0)}
            for fill in self.get("fills", []):
                if fill["type"] == "GRADIENT_LINEAR":
                    gradient_stops = fill.get("gradientStops", [])
                    gradient_pos = fill.get("gradientHandlePositions", [])

                    begin_pos = gradient_pos[0]
                    end_pos = gradient_pos[1]

                    # Map to Flet's Alignment (x, y)
                    begin = f"ft.Alignment({round(end_pos["x"],2)}, {round(end_pos["y"],2)})"
                    end = f"ft.Alignment({round(begin_pos["x"],2)}, {round(begin_pos["y"],2)})"
                    # print(begin, end)
                    hex_colors = [
                        f"""ft.Colors.with_opacity({round(color['a'],3)}, "#{int(color['r'] * 255):02x}{int(color['g'] * 255):02x}{int(color['b'] * 255):02x}")"""
                        for stop in gradient_stops
                        for color in [stop["color"]]
                    ]

                    if len(gradient_stops) < 2:
                        raise ValueError("Gradient must have at least two stops.")

                    effects["gradient"] = {
                        "type": fill["type"],
                        "colors": hex_colors,
                        "stops": [stop["position"] for stop in gradient_stops],
                        "begin": begin,
                        "end": end,
                    }

        except KeyError:
            pass
        return effects

    @property
    def corner_radius(self):
        return self.node.get("cornerRadius")

    @property
    def rectangle_corner_radii(self):
        return self.node.get("rectangleCornerRadii")

    def to_code(self):
        effects = self.get_effects()
        gradient_str = ""
        if effects["gradient"]:
            gradient = effects["gradient"]
            if gradient["type"] == "GRADIENT_LINEAR":
                gradient_str = f"""
                gradient=ft.LinearGradient(
                    colors=[{", ".join(gradient['colors'])}],
                    # stops={gradient["stops"]},
                    begin={gradient["begin"]},
                    end={gradient["end"]},
                    rotation=3.1415
                )
                """

        # Shadow to flet compatible str
        shadow_str = ""
        if effects["shadow"]:
            shadow = effects["shadow"]
            shadow_str = f"""
            shadow=ft.BoxShadow(
                spread_radius=2,
                blur_radius={shadow["blur"]//5},
                offset=ft.Offset({shadow["offset_x"]}, {shadow["offset_y"]}),
                color=ft.Colors.with_opacity(0.1,"{shadow["color"]}")
            ),
            """
        # blur to flet compatible str
        blur_str = ""
        if effects["background_blur"]:
            blur = effects["background_blur"]
            blur_str = f"blur={blur['radius']//2},"

        return f"""
        ft.Container(
            left={self.x},
            top={self.y},
            width={self.width},
            height={self.height},
            {blur_str}
            {shadow_str}
            border_radius={self.corner_radius},
            bgcolor=ft.Colors.with_opacity({self.opacity},"{self.bg_color}"),
            {gradient_str}
            )
"""


class Text(Vector):
    def __init__(self, node, frame):
        super().__init__(node)
        self.x, self.y = self.position(frame)
        self.width, self.height = self.size()

        self.text_opacity, self.text_color = self.color()

        self.font_family, self.font_size, self.font_weight = self.font_property()

        if "\n" in self.characters:
            self.text = f'"""{self.characters.replace("\n", "\\n")}"""'
        else:
            self.text = f'"{self.characters}"'

        self.text_align = self.style["textAlignHorizontal"]

    @property
    def characters(self) -> str:
        string: str = self.node.get("characters")
        text_case: str = self.style.get("textCase", "ORIGINAL")

        if text_case == "UPPER":
            string = string.upper()
        elif text_case == "LOWER":
            string = string.lower()
        elif text_case == "TITLE":
            string = string.title()

        return string

    @property
    def style(self):
        return self.node.get("style")

    @property
    def style_override_table(self):
        return self.node.get("styleOverrideTable")

    def font_property(self):
        style = self.node.get("style")

        font_name = style.get("fontPostScriptName")
        if font_name is None:
            font_name = style["fontFamily"]

        # TEXT- Weight
        font_weight = style.get("fontWeight")
        if font_weight:
            font_weight = f"w{font_weight}"

        font_name = font_name.replace("-", " ")
        font_size = style["fontSize"]

        return font_name, font_size, font_weight

    def to_code(self):
        return f"""
        ft.Container(
            content=ft.Text(value={self.text}, size={self.font_size}, color="{self.text_color}",weight="{self.font_weight}",font_family="{self.font_family}",text_align=ft.TextAlign.{self.text_align}),
            left={self.x},
            top={self.y},
            )
        """


class TextField(Vector):
    def __init__(self, node, frame):
        super().__init__(node)

        self.x, self.y = self.position(frame)
        self.width, self.height = self.size()
        self.border_opacity, self.border_color = self.strockes_color()
        self.border_width = int(self.node["strokeWeight"])
        self.opacity, self.bg_color = self.color()

        self.border_radius = self.get("cornerRadius", 0)

    def text_color_from_bg(self, bg_color):
        # Assuming bg_color is a hex string like "#RRGGBB"
        bg_color = bg_color.lstrip("#")

        # hex to RGB
        r, g, b = int(bg_color[0:2], 16), int(bg_color[2:4], 16), int(bg_color[4:6], 16)

        # luminance
        luminance = (0.299 * r + 0.587 * g + 0.114 * b) / 255

        # Choose text color based on luminance
        return "#FFFFFF" if luminance < 0.5 else "#000000"

    def to_code(self):

        content_pad = int(self.height - (self.height / 1.5)) / 2

        return f"""
        ft.Container(
            content=ft.TextField(
                width={self.width},
                height={self.height},
                border=ft.border.all({self.border_width}, "{self.border_color}"),
                border_radius={self.border_radius},
                bgcolor=ft.Colors.with_opacity({self.opacity},"{self.bg_color}"),
                cursor_height={self.height/1.5},
                cursor_color="{self.text_color_from_bg(self.bg_color)}",
                focused_border_color="{self.border_color}",
                content_padding={content_pad},
                text_style=ft.TextStyle(color="{self.text_color_from_bg(self.bg_color)}"),

                ),
            left={self.x},
            top={self.y}, )
"""


class Image(Vector):
    def __init__(self, node, frame, image_path, *, id_):
        super().__init__(node)

        self.x, self.y = self.position(frame)
        self.width, self.height = self.size()

        self.image_path = image_path
        self.id_ = id_

    def to_code(self):
        return f"""
ft.Image(
    src="{self.image_path}",left={self.x},top={self.y},width={self.width},height={self.height})

"""


class UnknownElement(Vector):
    def __init__(self, node, frame):
        super().__init__(node)
        self.x, self.y = self.position(frame)
        self.width, self.height = self.size()

    def to_code(self):
        return f"""
ft.Container(
    left={self.x},
    top={self.y},
    width={self.width},
    height={self.height},
    bgcolor="pink")
"""
